/*
SCARF Association Rule Mining Program
Copyright (c) 2012-13 Balazs Szalkai

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "chisquared.hpp"

#include "asa239.hpp"
#include <cmath>
#include <stdexcept>

double ChiSquared::pValue(const double degreesOfFreedom, const double x)
{
    if (std::isnan(degreesOfFreedom) || std::isnan(x)) return 1.0;
    if (x >= degreesOfFreedom * 1000) return 0.0;

    int dummy;
    double cdf = gammad(x * 0.5, degreesOfFreedom * 0.5, &dummy);
    if (dummy != 0 || std::isnan(cdf) || std::isinf(cdf)) return 1.0;

    return 1.0 - cdf;
}
