/*
SCARF Association Rule Mining Program
Copyright (c) 2012-13 Balazs Szalkai

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#pragma once

#include <ostream>
#include "tinythread.h"

class ConcurrentStream
{
    private: std::ostream& underlying;
    private: mutable tthread::mutex mutex;

    public: ConcurrentStream(std::ostream& _underlying)
        : underlying(_underlying)
    {
    }

    public: template<class T>
    const ConcurrentStream& operator<<(const T& value) const
    {
        tthread::lock_guard<tthread::mutex> lock(mutex);
        underlying<<value;
        return *this;
    }

    public: const ConcurrentStream& operator<<(std::ostream& (*value)(std::ostream&)) const
    {
        tthread::lock_guard<tthread::mutex> lock(mutex);
        underlying<<value;
        return *this;
    }

    public: const ConcurrentStream& operator<<(std::basic_ios<char>& (*value)(std::basic_ios<char>&)) const
    {
        tthread::lock_guard<tthread::mutex> lock(mutex);
        underlying<<value;
        return *this;
    }

    public: const ConcurrentStream& operator<<(std::ios_base& (*value)(std::ios_base&)) const
    {
        tthread::lock_guard<tthread::mutex> lock(mutex);
        underlying<<value;
        return *this;
    }

    public: void flush() const
    {
        tthread::lock_guard<tthread::mutex> lock(mutex);
        underlying.flush();
    }
};
