/*
SCARF Association Rule Mining Program
Copyright (c) 2012-13 Balazs Szalkai

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#pragma once

#include <cstdint>
#include "nullable.hpp"
#include "range.hpp"
#include <string>
#include <vector>
#include "xml.hpp"

class Mine;

/// Combines OR'ed elements thru AND operation
class ConjunctiveNormalForm
{
    public: enum class Operator
    {
        And, Or
    };

    /// A Node represents an elementary clause of a CNF.
    public: class Node
    {
        /// The operator following this node.
        /// Can be anything for the last node.
        public: Operator op;
        /// A column index in a Mine object. Null means that the node is unused.
        public: Nullable<size_t> column;

        public: Node(Operator _op)
            : op(_op), column()
        {
        }
    };

	public: std::vector<Node> nodes;

	public: void createFromPattern(const std::vector<size_t>& orClauseSizes);
    public: void parsePattern(const std::string &pattern, bool sort);

    /// Gets the index ranges corresponding to OR clauses.
    public: std::vector<Range<size_t>> getPattern() const;

	/// This hash is insensitive to permuting the children,
	/// as long as it results in an equivalent CNF.
	public: std::int64_t getHash() const;
	public: std::string toString(const Mine& mine) const;
	public: void toXml(XmlWriter& xml, const Mine &mine) const;
};
