/*
SCARF Association Rule Mining Program
Copyright (c) 2012-13 Balazs Szalkai

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "pathutils.hpp"

#include <algorithm>
#include "utils.hpp"

std::string combinePath(const std::string& currentFolder, const std::string& filename)
{
    if (isAbsolutePath(filename) || currentFolder.size() == 0) return filename;
    return includeTrailingPathDelimiter(currentFolder) + filename;
}

std::string extractPath(const std::string& filename)
{
    std::string::const_iterator it =
        std::find_if(filename.rbegin(), filename.rend(), isPathDelimiter).base();
    return std::string(filename.begin(), it);
}

std::string includeTrailingPathDelimiter(const std::string& path)
{
    if (path.size() == 0) return "/";
    if (isPathDelimiter(path.back())) return path;
    return path + '/';
}

bool isAbsolutePath(const std::string& filename)
{
    size_t n = filename.size();
    if (n == 0) return false;
    if (isPathDelimiter(at(filename, 0))) return true;
    if (n >= 2 && at(filename, 1) == ':') return true;
    return false;
}

bool isPathDelimiter(char c)
{
    return c == '/' || c == '\\';
}
