/*
SCARF Association Rule Mining Program
Copyright (c) 2012-13 Balazs Szalkai

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#pragma once

#include <stdexcept>

template <class T>
class Range
{
    private: T f_begin;
    private: T f_end;

    public: Range() {}

    /// \param _begin The beginning of the range (inclusive).
    /// \param _end The end of the range (non-inclusive).
    public: Range(const T& _begin, const T& _end)
        : f_begin(_begin), f_end(_end)
    {
    }

    public: void setBegin(const T& value)
    {
        f_begin = value;
    }
    public: void setEnd(const T& value)
    {
        f_end = value;
    }

    public: class Iterator
    {
        private: T value;

        public: Iterator(const T _value)
            : value(_value)
        {
        }

        public: T operator*() const
        {
            return value;
        }

        public: Iterator& operator++()
        {
            ++value;
            return *this;
        }

        public: bool operator==(const Iterator& other)
        {
            return value == other.value;
        }

        public: bool operator!=(const Iterator& other)
        {
            return value != other.value;
        }
    };

    public: Iterator begin() const
    {
        return f_begin;
    }

    public: Iterator end() const
    {
        return f_end;
    }

    public: T front() const
    {
        if (size() == 0)
            throw std::out_of_range("Range is empty.");
        return f_begin;
    }

    public: T back() const
    {
        if (size() == 0)
            throw std::out_of_range("Range is empty.");
        return f_end - 1;
    }

    public: T size() const
    {
        return f_end - f_begin;
    }

    public: T at(const T index) const
    {
        if (index >= size())
            throw std::out_of_range("Index out of range.");
        return f_begin + index;
    }
};
