/*
SCARF Association Rule Mining Program
Copyright (c) 2012-13 Balazs Szalkai

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "xml.hpp"

#include <stdexcept>

XmlWriter::XmlWriter(std::ostream& outputBuffer)
: buffer(outputBuffer)
{
    buffer<<"<?xml version=\"1.0\" encoding=\"UTF-8\"?>"<<std::endl;
    canAddAttributes = false;
}

void XmlWriter::finishAddingAttributes()
{
    if (canAddAttributes)
    {
        canAddAttributes = false;
        buffer<<'>';
    }
}

void XmlWriter::start(const std::string& elementName)
{
    finishAddingAttributes();
    buffer<<'<'<<elementName;
    elementStack.push_back(elementName);
    canAddAttributes = true;
}

void XmlWriter::attr(const std::string& key, const std::string& value)
{
    if (!canAddAttributes)
        throw std::invalid_argument("Start element has already been closed.");
    buffer<<' '<<key<<"=\"";
    xmlEscape(buffer, value);
    buffer<<"\"";
}

void XmlWriter::text(const std::string& text)
{
    finishAddingAttributes();
    xmlEscape(buffer, text);
}

void XmlWriter::end()
{
    finishAddingAttributes();
    if (elementStack.empty())
        throw std::invalid_argument("No element to end.");
    buffer<<"</"<<elementStack.back()<<'>';
    elementStack.pop_back();
}

void XmlWriter::xmlEscape(std::ostream& output, const std::string& s)
{
    for (char c : s)
    {
        switch (c)
        {
            case '"': output<<"&quot;"; break;
            case '\'': output<<"&apos;"; break;
            case '<': output<<"&lt;"; break;
            case '>': output<<"&gt;"; break;
            case '&': output<<"&amp;"; break;
            default: output<<c; break;
        }
    }
}
