/*
SCARF Association Rule Mining Program
Copyright (c) 2012-13 Balazs Szalkai

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#pragma once

#include <ostream>
#include <string>
#include <vector>

class XmlWriter
{
    private: std::ostream& buffer;
    private: bool canAddAttributes;
    private: std::vector<std::string> elementStack;

    public: XmlWriter(std::ostream& outputBuffer);

    private: void finishAddingAttributes();

    /// Starts an XML element. Attributes can be added after that, by calling attr()
    public: void start(const std::string& elementName);
    /// Adds an attribute to an element.
    public: void attr(const std::string& key, const std::string& value);
    /// Outputs text.
    public: void text(const std::string& text);
    /// Emits a closing element.
    public: void end();

    /// Replaces special characters with xml entities.
    public: static void xmlEscape(std::ostream& output, const std::string& s);
};
